
-- VHDL Test Bench 
-- Testbench that reads operands from a file and 
-- Check the division result
--

LIBRARY  IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE IEEE.std_logic_unsigned.all;
LIBRARY ieee;
USE IEEE.STD_LOGIC_TEXTIO.ALL;
USE STD.TEXTIO.ALL;

ENTITY testbench_file IS
END testbench_file;

ARCHITECTURE behavior OF testbench_file IS 

constant CLK_LENGTH: time := 200 ns; --clk periodo de reloj

file stimuli_file : text is in "test.txt"; 
file results_file: text is out "results.txt";

  COMPONENT div_seq_inst
  PORT(
      a : IN std_logic_vector(255 downto 0);
      b : IN std_logic_vector(127 downto 0);
      clk : IN std_logic;          
      ini: in STD_LOGIC;    --initialization, capture input data
      done: out STD_LOGIC;   --end signal
      q : OUT std_logic_vector(255 downto 0);
      r : OUT std_logic_vector(127 downto 0)
  );
  END COMPONENT;

  SIGNAL x :  std_logic_vector(255 downto 0);
  SIGNAL y :  std_logic_vector(127 downto 0);
  SIGNAL clk, ini, done :  std_logic;
  SIGNAL q :  std_logic_vector(255 downto 0);
  SIGNAL r :  std_logic_vector(127 downto 0);
  constant PERIOD: time := 100 ns;   
  constant SETUP_TIME: time := 10 ns;   

  signal aRES: std_logic_vector (255 downto 0);


BEGIN

  uut: div_seq_inst PORT MAP(
    a => x, b => y,
    clk => clk,
    ini => ini, done => done,
    q => q, r => r
  );
    
  PROCESS -- clock process (drives clk),
  BEGIN
    clk <= transport '0';
    WAIT FOR SETUP_TIME;
    clk <= transport '1';
    WAIT FOR PERIOD/2;
    clk <= transport '0';
    WAIT FOR PERIOD/2 - SETUP_TIME;
  END PROCESS;

  read_file: process --lectura del fichero
    VARIABLE TX_OUT : LINE;
    VARIABLE TX_ERROR : INTEGER := 0;
  
  procedure char_2_hex(char: in character; vector: out std_logic_vector (3 downto 0)) is
  variable result: natural := 0;
  begin 
      if ('0' <= char and char <= '9') then
        result:= character'pos(char)-character'pos('0');
      elsif 'a' <= char and char <= 'f' then
        result := character'pos(char)-character'pos('a')+10;
      elsif 'A' <= char and char <= 'F' then
        result := character'pos(char)-character'pos('A')+10;
      else
        report "HEX convertion error" severity error;
      end if;
      vector(3 downto 0) := CONV_STD_LOGIC_VECTOR(result,4);
  end char_2_hex;  
  procedure text_2_stdlv256(str: in string (256/4+1 downto 1); vector: out std_logic_vector (255 downto 0)) is
  variable parcial_result: std_logic_vector(3 downto 0);
  begin 
    for i in 256/4 downto 1 loop
      char_2_hex(str(i),parcial_result );
      vector((i*4-1) downto (i-1)*4) := parcial_result;
    end loop;
  end text_2_stdlv256;
  
  procedure text_2_stdlv128(str: in string (128/4+1 downto 1); vector: out std_logic_vector (127 downto 0)) is
  variable parcial_result: std_logic_vector(3 downto 0);
  begin 
    for i in 128/4 downto 1 loop
      char_2_hex(str(i),parcial_result );
      vector((i*4-1) downto (i-1)*4) := parcial_result;
    end loop;
  end text_2_stdlv128;

  VARIABLE TX_LOC : LINE;
  VARIABLE TX_STR : String(1 to 4096);
  variable line : line; 
  variable index : integer;
  variable op_A, res_Q: string (256/4+1 downto 1);
  variable op_B, res_R: string (128/4+1 downto 1);
  variable XX, QQ, RES: std_logic_vector (255 downto 0);
  variable YY, RR: std_logic_vector (127 downto 0);
  variable Y_x_Q: std_logic_vector (256+127 downto 0);

  begin
    wait for 4*period;
    readline(stimuli_file, line);
    while (not endfile(stimuli_file))loop
        readline(stimuli_file, line); 
        read (line, index);
        read (line, op_A);
        read (line, op_B);
        
        text_2_stdlv256(op_A, XX);
        X <= XX;
        text_2_stdlv128(op_B, YY);
        Y <= YY;
        ini <= '1';
        wait for period;
        ini <= '0';
        wait until done = '1';
        wait for period;
        Y_x_Q := Y*Q;
        RES := Y_x_Q(255 downto 0) + R;
        aRES <= RES;
        IF (RES /= X) THEN 
           write(TX_LOC,string'("ERROR!!! X=")); write(TX_LOC, X);
           write(TX_LOC,string'(" Y=")); write(TX_LOC, Y);
           write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
           write(TX_LOC,string'(" R=")); write(TX_LOC, R);
           write(TX_LOC, string'(")"));
           TX_STR(TX_LOC.all'range) := TX_LOC.all;
           writeline(results_file, TX_LOC);
           Deallocate(TX_LOC);
           ASSERT (FALSE) REPORT TX_STR SEVERITY FAILURE;
        ELSIF (R > Y) THEN 
              write(TX_LOC,string'("--> Error Reminder Grater than Y =")); write(TX_LOC, 0.0);
              write(TX_LOC,string'("ns X=")); write(TX_LOC, X);
              write(TX_LOC,string'(" Y=")); write(TX_LOC, Y);
              write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
              write(TX_LOC,string'(" R=")); write(TX_LOC, R);
              write(TX_LOC, string'(" "));
              TX_STR(TX_LOC.all'range) := TX_LOC.all;
              writeline(results_file, TX_LOC);
              Deallocate(TX_LOC);
              ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
        END IF;
    end loop;
    ASSERT (FALSE) REPORT "Simulation OK (not errors)" SEVERITY FAILURE;
end process;

END;
